import pathlib

from meterviewer.generator.jsondb import gen_db

if __name__ == "__main__":
  for is_test in [True, False]:
    name = "test" if is_test else "train"
    gen_db(
      infile=pathlib.Path(__file__).parent / "config.toml",
      output=pathlib.Path(__file__).parent / f"meterdb_{name}.json",
      is_test=is_test,
    )



import streamlit as st

main_text = """
This tool-sets is built for quick process meter image data.

## Features

1. View Dataset

View the rectangle area of given image in dataset.

2. Download Configuration File

Download the configuration file of given image in dataset.

"""


def main():
  st.title("Meter data viewer")
  st.markdown(main_text)


if __name__ == "__main__":
  main()



from __future__ import annotations
import pathlib
import torch
from torch.utils.data import Dataset
import typing as t
import numpy as np
from torchvision import datasets
from PIL import Image
# from torchvision.transforms import transforms


class NumpyDataset(Dataset):
  def __init__(
    self,
    root: pathlib.Path,
    x_name: str | pathlib.Path,
    y_name: str | pathlib.Path,
    transform: t.Optional[t.Callable] = None,
    target_transform: t.Optional[t.Callable] = None,
  ):
    self.x_name = x_name
    self.y_name = y_name
    self.x = np.load(root / x_name)
    self.y = np.load(root / y_name)
    self.transform = transform
    self.target_transform = target_transform

  def __getitem__(self, index: int) -> tuple[Image.Image, np.ndarray]:
    x = self.x[index]
    y = self.y[index]

    x = Image.fromarray(x)

    if self.transform:
      x = self.transform(x)
    if self.target_transform:
      y = self.target_transform(y)
    return x, y



"""view all images."""

import pathlib
import typing as t

from matplotlib import pyplot as plt

from meterviewer import T

from ..img import process


def view_dataset_on_disk(name: str):
  def mmm(
    prefix_name: pathlib.Path,
    load_from_disk: t.Callable,
    view_dataset: t.Callable,
    show: bool = True,
    nums: int = 3,
  ):
    if not show:
      return

    assert prefix_name.exists()
    x = load_from_disk(prefix_name / name)
    x = process.np_to_img(x)
    view_dataset(nums, x)

  return mmm


def view_dataset_in_rows(num: int, imglist: T.ImgList):
  # 创建一个 GridSpec 实例，1行3列
  from matplotlib.gridspec import GridSpec

  fig = plt.figure(figsize=(15, 5))
  gs = GridSpec(1, 3, figure=fig)

  # 第一个子图
  ax1 = fig.add_subplot(gs[0, 0])
  ax1.imshow(imglist[0])

  # 第二个子图
  ax2 = fig.add_subplot(gs[0, 1])
  ax2.imshow(imglist[1])

  # 第三个子图
  ax3 = fig.add_subplot(gs[0, 2])
  ax3.imshow(imglist[2])

  plt.tight_layout()
  plt.show()


def view_dataset(num: int, imglist: T.ImgList):
  if imgshape := len(imglist[0].shape) > 3:
    raise Exception(f"imglist or Dataset meets error, shape: {imgshape}")
  for im in imglist[:num]:
    plt.imshow(im)
    plt.show()



import pathlib
import typing as t
import numpy as np
from meterviewer import files
from .utils import gen_img


def test_md5():
  res = files.compute_md5("src/tests/for_md5")
  assert res == "5093b7d2c218eaebde04b58b20491ef6"


def test_reshape():
  img = gen_img(size=(35, 25, 3))
  assert files.transform_img(img).shape == (1, 35, 25, 3)

  # test concat
  im1 = files.transform_img(img)
  im2 = files.transform_img(img)
  assert np.vstack([im1, im2, im2, im2]).shape == (4, 35, 25, 3)


def test_transform_label():
  label = ["1", "2", "3", "4", "5"]
  assert files.transform_label(label).shape == (1, 5)


def test_scan_pics(root_path):
  path = root_path / pathlib.Path(r"lens_6/XL/XL/M1L1XL/Digit/0")
  count = 0
  pics: t.List[pathlib.Path] = []

  def update_count(name):
    nonlocal count, pics
    pics.append(name)
    count += 1

  [update_count(name) for name in files.scan_pics(path)]
  assert count == 20
  assert pics[0].parent.name == "0"
  # path like this:
  # PosixPath('/home/svtter/Work/Dataset/MeterData/lens_6/XL/XL/M1L1XL/Digit/0/4_2018-11-23-19-21-01.jpg')
  # assert False, pics



# quick view of a dataset

import glob
import os

# import pathlib
# import numpy as np
# import pandas as pd
# from matplotlib import pyplot as plt
import streamlit as st

from meterviewer.config import get_root_path
from meterviewer.datasets import imgv
from meterviewer.datasets.read import config
from meterviewer.img import cut, draw

desc = """
用于快速查看图片以及矩形框
"""


def main():
  st.title("Quick Viewer of Meter dataset")
  st.markdown(desc)
  # data_path = pathlib.Path("/home/xiuhao/work/Dataset/MeterData/lens_5/XL/DATA")
  data_path = get_root_path()

  dataset_len_ = st.number_input("Dataset length", value=6)
  dataset_type = st.text_input("Dataset type", value="XL")

  # dataset_name = st.text_input("Dataset name", value="M4L1XL")
  dataset_folder = data_path / f"lens_{dataset_len_}" / dataset_type / dataset_type
  dataset_name = st.selectbox("Dataset name", options=os.listdir(dataset_folder))

  img_folder = dataset_folder / dataset_name
  img_list = glob.glob(str(img_folder / "*.jpg"))
  # st.text(os.listdir(img_folder))

  filename = st.selectbox("Choose a file", img_list)
  # filename = st.text_input("Enter the filename", value="2018-11-23-12-16-01.jpg")

  # if st.button("View"):
  im, v, rect = imgv.view_one_img_v(img_folder / filename)
  st.text(f"image shape: {im.shape}")
  st.image(im, caption=f"filename: {filename}")

  st.text("with rect")
  rect_im = draw.draw_rectangle(im, rect)
  st.image(rect_im, caption=f"with rect: {filename}")

  cutted = cut.cut_img(im, rect)
  st.image(cutted, caption=f"cutted: {filename}")

  xml_path = config.get_xml_config_path(img_folder / filename, "block")
  st.download_button(
    label="Download xml",
    data=open(xml_path, "rb"),
    file_name=xml_path.name,
    mime="application/xml",
  )


main()



from meterviewer.img import process
from PIL import Image


def test_gen_pics():
    im = process.gen_empty_im((30, 120, 3))
    im = Image.fromarray(im)
    filename = "/tmp/test.png"
    im.save(filename)
    process.show_img(im, is_stop=False)



import streamlit as st
import pandas as pd
import pathlib
import numpy as np
import os
import ast
import shutil


def main():
  # st.Page("copy page", title="Tool to backup numpy")
  st.title("Tool to backup numpy.")
  root_path = pathlib.Path("/home/xiuhao/work/Dataset/HRCdata")
  root_path = st.text_input("Enter the root path", value=str(root_path))
  root_path = pathlib.Path(root_path)

  folder = st.text_input("Enter the path of the data", value="Train")

  st.text(os.listdir(root_path / folder))

  x_name = st.text_input("Enter the value of x_name", value="x_train.npy")
  y_name = st.text_input("Enter the value of y_name", value="y_train.npy")

  new_x_name = st.text_input(
    "Enter the value of new_x_name", value="x_train.original.npy"
  )
  new_y_name = st.text_input(
    "Enter the value of new_x_name", value="y_train.original.npy"
  )

  if st.button("Confirm"):
    shutil.copy(root_path / folder / x_name, root_path / folder / new_x_name)
    shutil.copy(root_path / folder / y_name, root_path / folder / new_y_name)


main()



import pathlib
from meterviewer.models import func
import typing as t


def from_db() -> t.Generator[t.Tuple[pathlib.Path, pathlib.Path, str], None, None]:
  dbpath = "alldata.db"
  items = func.get_carry_items(dbpath)
  items = list(items)
  for item in items:
    yield (
      pathlib.Path(item.filename),
      pathlib.Path(item.filename).parent,
      str(id),
    )



from __future__ import annotations

import functools
import pathlib
import random
import sys
import typing as t

import toml
from loguru import logger

from meterviewer import T, files
from meterviewer.config import get_root_path
from meterviewer.datasets import dataset
from meterviewer.datasets.read import single
from meterviewer.img import process

# 设置控制台输出的日志级别为 WARNING
logger.remove()  # 移除默认的控制台输出
logger.add(sys.stdout, level="INFO")


getList = t.Literal["dataset", "path", "length", "total_nums"]


def load_config(config_path: pathlib.Path) -> t.Callable[[getList], t.Any]:
  data: t.Optional[dict] = None
  dataset_list: t.List[str] = []

  def load_conf() -> dict:
    nonlocal data
    if data is None:
      with open(config_path, "r") as f:
        data = toml.load(f)
    assert data is not None, (config_path, data)
    return data

  def get_dataset() -> str:
    """随机选择一个数据集"""
    nonlocal dataset_list
    dataset_list = get_config("dataset")
    return random.choice(dataset_list)

  def get_config(name: str):
    c = load_conf().get("generate_config", None)
    if c is None:
      raise Exception('config "generate_config" not found')
    return c.get(name)

  pt = functools.partial
  get_path = pt(get_config, name="path")
  get_length = pt(get_config, name="length")
  get_total_nums = pt(get_config, name="total_nums")

  def get_func(name: getList) -> t.Callable:
    func_map: t.Mapping[str, t.Callable] = {
      "dataset": get_dataset,
      "path": get_path,
      "length": get_length,
      "total_nums": get_total_nums,
    }
    return func_map[name]

  return get_func


def generate_dataset(config_path: pathlib.Path):
  root_path = get_root_path()
  get_f = load_config(config_path=config_path)

  def read_rand_img(digit: int | str) -> T.NpImage:
    return single.read_rand_img(
      digit=digit,
      root=root_path,
      get_dataset=get_f("dataset"),
      promise=True,
    )

  def gen_block(digit: T.DigitStr) -> T.NpImage:
    return dataset.generate_block_img(digit, dataset.join_with_resize, read_rand_img)

  generated_path = root_path / pathlib.Path(get_f("path")())
  generated_path.mkdir(exist_ok=True)

  filesave = functools.partial(
    files.save_img_labels_with_default,
    prefix_name=generated_path,
    save_to_disk=files.save_to_disk,
  )

  def check_imgs(imglist):
    size = imglist[0].shape
    imgs = process.resize_imglist(imglist)
    for im in imgs:
      assert size == im.shape

  create_dataset = dataset.create_dataset_func(check_imgs=lambda x: None, total=9)
  imgs, labels = create_dataset(
    length=get_f("length")(),
    nums=get_f("total_nums")(),
    gen_block_img=gen_block,
  )
  filesave(imgs, labels)


def main():
  config_list = [
    "dataset-gen.toml",
    "dataset-gen-2.toml",
    "dataset-gen-3.toml",
    "dataset-gen-4.toml",
    "dataset-gen-5.toml",
  ]
  parent = pathlib.Path(__file__).parent
  for config in config_list:
    logger.info(f"generating dataset with {config}...")
    generate_dataset(parent / config)



from __future__ import annotations

import functools
import pathlib
import random
import sys
import typing as t

import toml
from loguru import logger

from meterviewer import T, files
from meterviewer.config import get_root_path
from meterviewer.datasets import dataset
from meterviewer.datasets.read import single
from meterviewer.img import process

getList = t.Literal["dataset", "path", "length", "total_nums"]


class DatasetGenerator(object):
  """class format definition of dataset generator"""

  def __init__(self, config_path: pathlib.Path) -> None:
    self.root_path = get_root_path()
    self.config_path = config_path
    self.get_f = self.load_config(config_path=self.config_path)

  def load_config(self, config_path: pathlib.Path) -> t.Callable[[getList], t.Any]:
    data: t.Optional[dict] = None
    dataset_list: t.List[str] = []

    def load_conf() -> dict:
      nonlocal data
      if data is None:
        with open(config_path, "r") as f:
          data = toml.load(f)
      assert data is not None, (config_path, data)
      return data

    def get_dataset() -> str:
      nonlocal dataset_list
      dataset_list = get_config("dataset")
      return random.choice(dataset_list)

    def get_config(name: str):
      c = load_conf().get("generate_config", None)
      if c is None:
        raise Exception('config "generate_config" not found')
      return c.get(name)

    pt = functools.partial
    get_path = pt(get_config, name="path")
    get_length = pt(get_config, name="length")
    get_total_nums = pt(get_config, name="total_nums")

    def get_func(name: getList) -> t.Callable:
      func_map: t.Mapping[str, t.Callable] = {
        "dataset": get_dataset,
        "path": get_path,
        "length": get_length,
        "total_nums": get_total_nums,
      }
      return func_map[name]

    return get_func

  def read_rand_img(self, digit: int | str) -> T.NpImage:
    return single.read_rand_img(
      digit=digit,
      root=self.root_path,
      get_dataset=self.get_f("dataset"),
      promise=True,
    )

  def gen_block(self, digit: T.DigitStr) -> T.NpImage:
    return dataset.generate_block_img(
      digit, dataset.join_with_resize, self.read_rand_img
    )

  def main(self):
    generated_path = self.root_path / pathlib.Path(self.get_f("path")())
    generated_path.mkdir(exist_ok=True)

    filesave = functools.partial(
      files.save_img_labels_with_default,
      prefix_name=generated_path,
      save_to_disk=files.save_to_disk,
    )

    def check_imgs(imglist):
      size = imglist[0].shape
      imgs = process.resize_imglist(imglist)
      for im in imgs:
        assert size == im.shape

    create_dataset = dataset.create_dataset_func(check_imgs=lambda x: None, total=9)
    imgs, labels = create_dataset(
      length=self.get_f("length")(),
      nums=self.get_f("total_nums")(),
      gen_block_img=self.gen_block,
    )
    filesave(imgs, labels)



"""create jsondb for meter-viewer"""

import glob
import json
import pathlib
import random
import typing as t

import toml

from meterviewer.datasets.read.detection import read_image_area

from .schema import Item, MeterDB


# cache config function (only read disk once), returns get_random_dataset and load_conf
def load_config(config_path: pathlib.Path) -> t.Callable[[], dict]:
  data: t.Optional[dict] = None

  def load_conf() -> dict:
    nonlocal data
    if data is None:
      with open(config_path, "r") as f:
        data = toml.load(f)
    assert data is not None, (config_path, data)
    return data

  return load_conf


get_local_config = None


# 随机选择一个数据集
def get_random_dataset(is_train: bool = True) -> str:
  dataset_list = get_all_dataset(is_train)
  return random.choice(dataset_list)


# 获取数据集列表
def get_all_dataset(is_train: bool = True) -> list[str]:
  config = get_local_config()
  if is_train:
    key = "train_dataset"
  else:
    key = "test_dataset"
  return config["base"][key]


def get_base_dir() -> str:
  """获取数据集的 base_dir"""
  config = get_local_config()
  return config["base"]["root_path"]


def get_mid_path(is_test: bool = False) -> str:
  """获取数据集的 mid_path"""
  if is_test:
    return "lens_6/CS/all_CS"
  else:
    return "lens_6/XL/XL"


def get_random_data(is_test: bool = False) -> pathlib.Path:
  """随机获取一个数据集下的图片"""
  dataset = get_random_dataset(is_train=not is_test)
  base_dir = get_base_dir()
  mid_path = get_mid_path(is_test=is_test)

  data_path = glob.glob(str(pathlib.Path(base_dir) / mid_path / dataset / "*.jpg"))
  return random.choice(data_path)


def set_local_config(infile: pathlib.Path):
  """设置本地配置"""
  global get_local_config
  get_local_config = load_config(config_path=infile)


def gen_db(infile: pathlib.Path, output: pathlib.Path, is_test: bool = False):
  """读取数据集下所有的图片，以及点的位置，生成一个json文件"""

  # set the get_local_config
  set_local_config(infile)

  data = []
  mid_path = get_mid_path(is_test)

  for dataset in get_all_dataset(is_train=not is_test):
    base_dir = get_base_dir()
    data_path = glob.glob(str(pathlib.Path(base_dir) / mid_path / dataset / "*.jpg"))
    for jpg_data in data_path:
      rect = read_image_area(pathlib.Path(jpg_data))
      item = Item(
        filepath=jpg_data,
        dataset=dataset,
        xmin=rect["xmin"],
        xmax=rect["xmax"],
        ymin=rect["ymin"],
        ymax=rect["ymax"],
      )
      data.append(item)

  meter_db = MeterDB(data=data)

  with open(output, "w") as f:
    json.dump(meter_db.model_dump(), f)

  return output



"""create new generated dataset, generate all suitable images."""

import typing as t
from pathlib import Path as P

from PIL import Image

from meterviewer import T, files
from meterviewer.datasets import dataset, imgv
from meterviewer.datasets.read import config, single
from meterviewer.img import cut, process


def cut_one_img(filepath: P) -> t.Tuple[T.ImgList, T.DigitStr]:
  im, val, pos = imgv.view_one_img_v(filepath)
  xml_path = config.get_xml_config_path(filepath, "single")
  pos_list = config.read_rect_from_file(xml_path, "single")

  assert isinstance(pos_list, t.List)
  im_list = []
  for pos in pos_list:
    im1 = cut.cut_img(im, rect=pos.to_dict())
    im_list.append(im1)
  return im_list, list(val)


SaveFunc = t.Callable[[T.NpImage, str, int], t.Any]


def create_save_func(dataset_path: P, original_filepath: P) -> SaveFunc:
  assert dataset_path.exists(), f"the dataset: {dataset_path} should exist"

  def save_to_disk(im: T.NpImage, val: str, i: int):
    folder = dataset_path / val
    folder.mkdir(exist_ok=True)
    Image.fromarray(im).save(folder / f"{original_filepath.stem}_{i}.png")

  return save_to_disk


def cut_save_one(root_path: P, filepath: P):
  def cut_save(filepath: P, save_to_disk: SaveFunc):
    """切割图片并保存到磁盘上."""
    im_list, val = cut_one_img(filepath)
    for i, im in enumerate(im_list):
      save_to_disk(im, val[i], i)

  cut_save(filepath, create_save_func(root_path / "./generated", filepath))



import pathlib
import typing as t
import numpy as np
from . import types as T
import hashlib

import toml


def use_smart_name(dataset: pathlib.Path) -> T.NameFunc:
  """
  Determine the appropriate file names for X and Y data in a dataset.
  Returns a function that provides the correct file names.
  """
  x_name_list = ["x_train.npy", "x_all.npy", "x.npy"]
  y_name_list = ["y_train.npy", "y_all.npy", "y.npy"]

  for x, y in zip(x_name_list, y_name_list):
    if (dataset / pathlib.Path(x)).exists() and (dataset / pathlib.Path(y)).exists():

      def get_x_y():
        return x, y

      return get_x_y
  else:
    raise Exception("No valid npy files found")


def compute_md5(file_path, chunk_size=8192):
  """
  Compute the MD5 hash of a file.

  Args:
      file_path (str): The path to the file.
      chunk_size (int): The size of each chunk to read from the file.

  Returns:
      str: The MD5 hash of the file in hexadecimal format.
  """
  md5_hash = hashlib.md5()

  with open(file_path, "rb") as f:
    while chunk := f.read(chunk_size):
      md5_hash.update(chunk)

  return md5_hash.hexdigest()


def read_toml(filename: pathlib.Path) -> t.Optional[t.Dict]:
  """
  Read and parse a TOML file, returning its contents as a dictionary.
  """
  try:
    with open(filename, "r") as f:
      data = toml.load(f)
      print(f"Read '{filename}' successfully.")
      return data
  except Exception as e:
    print(f"Error to read '{filename}': {e}")


def write_toml(filename: pathlib.Path, data):
  """
  Write data to a TOML file.
  """
  try:
    with open(filename, "w") as f:
      toml.dump(data, f)
    print(f"Data written to '{filename}' successfully.")
  except Exception as e:
    print(f"Error writing to '{filename}': {e}")


def write_shape(img: T.ImgList, nums: int = 3):
  """
  Write the shapes of the first 'nums' images to a debug log file.
  """
  debug_file = pathlib.Path("debug.log")
  with open(debug_file, "w") as f:
    for i in range(nums):
      f.write(f"img_{i} shape is: {img[i].shape}\n")


def scan_pics(path: pathlib.Path) -> t.Iterator[pathlib.Path]:
  """
  Scan a directory for image files (jpg, png, jpeg) and yield their paths.
  """
  for p in path.iterdir():
    if p.suffix in [".jpg", ".png", ".jpeg"]:
      yield p


def transform_img(img: T.NpImage) -> T.NpImage:
  """
  Reshape a single image by adding a batch dimension.
  """
  return np.expand_dims(img, axis=0)


def transform_label(label: T.DigitStr, to_int: bool) -> T.Label:
  """
  Transform a label string into a numpy array, optionally converting to integers.
  """
  if to_int:
    label_ = [int(i) for i in label]
  else:
    label_ = label
  return np.expand_dims(np.array(label_), axis=0)


def save_imgs_labels(
  imgs_labels: t.Tuple[t.List[T.NpImage], t.List[T.DigitStr]],
  np_name: t.Callable[[], t.Tuple[pathlib.Path, str, str]],
  save_to_disk: t.Callable,
):
  """
  Save a list of images and labels to disk as numpy arrays.
  """
  imgs, labels = imgs_labels
  imgs = [np.expand_dims(img, axis=0) for img in imgs]
  labels_ = [transform_label(label, False) for label in labels]
  x_train = np.vstack(imgs)
  y_train = np.vstack(labels_)

  prefix_path, x_name, y_name = np_name()
  save_to_disk(str(prefix_path / x_name), x_train)
  save_to_disk(str(prefix_path / y_name), y_train)


def save_img_labels_with_default(
  imgs: t.List[T.NpImage],
  labels: t.List[T.DigitStr],
  prefix_name: pathlib.Path,
  save_to_disk: t.Callable,
):
  """
  Save images and labels to disk using default file names.
  """

  def np_name():
    return prefix_name, T.x_name, T.y_name

  return save_imgs_labels(
    (imgs, labels),
    np_name,
    save_to_disk,
  )


def save_to_disk(filename: str, data: np.ndarray):
  """
  Save a numpy array to disk.
  """
  with open(filename, "wb") as f:
    np.save(f, data)


def load_from_disk(filename) -> np.ndarray:
  """
  Load a numpy array from disk.
  """
  with open(filename, "rb") as f:
    return np.load(f)


def load_from_disk_with_md5(filename, with_md5: str) -> t.Tuple[np.ndarray, str]:
  """
  Load a numpy array from disk and optionally compute its MD5 hash.
  """
  md5 = ""
  if with_md5:
    md5 = compute_md5(filename)
  return load_from_disk(filename), md5



from meterviewer import values
import pytest


def test_is_carry():
  assert values.is_carry("123") is False
  assert values.is_carry("1239")
  assert values.is_carry("1230")


def test_make_full():
  assert values.make_full([1, 2, 3], 6) == [False, False, False, 1, 2, 3]
  assert values.make_full([1, 2, 3], 3) == [1, 2, 3]
  assert values.make_full([], 3) == [False, False, False]

  with pytest.raises(ValueError):
    assert values.make_full([1, 2, 3], 2)


def test_get_carry_arr():
  assert values.get_carry_array(list("123132")) == [False] * 6
  assert values.get_carry_array(list("12909")) == [False, False, False, True, True]
  assert values.get_carry_array(list("12999")) == [False, True, True, True, True]



"""read data from disk.
处理单个数字数据集的函数，数据集格式为 dataset_name/[0-9]
"""

from __future__ import annotations

import pathlib
import random
import typing as t

from loguru import logger
from matplotlib import pyplot as plt

from meterviewer import T, files, func
from meterviewer.img import process

from ..dataset import get_dataset_path


def path_fusion(
  root: pathlib.Path,
  dataset_name: str,
  num: int,
):
  """生成单个数字图像的路径
  Args:
      root: 根目录路径
      dataset_name: 数据集名称
      num: 数字(0-9)
  Returns:
      对应数字图像的完整路径
  """
  p = get_dataset_path(root, dataset_name) / "Digit" / str(num)
  return p


def read_rand_img(
  root: pathlib.Path,
  get_dataset: t.Callable[[], t.Union[str, pathlib.Path]],
  digit: t.Union[int, str],
  promise=False,
) -> T.NpImage:
  """随机读取一张数字图像，单字
  Args:
      root: 根目录路径
      get_dataset: 获取数据集名称的函数
      digit: 要读取的数字或'x'(表示空白图像)
      promise: 是否确保路径存在
  Returns:
      随机选择的数字图像数组
  Raises:
      Exception: 当数据集中没有图像时抛出异常
  """
  if digit == "x":
    im = process.gen_empty_im((32, 40, 3))
    return im

  get_one = read_single_digit(
    root,
    get_dataset=get_dataset,
    num=int(digit),
    promise=promise,
  )
  all_imgs = list(get_one())
  length = len(all_imgs)

  if length == 0:
    raise Exception(f"Dataset contains no images, dataset: {get_dataset()}")

  i = random.randint(0, length - 1)
  im = plt.imread(all_imgs[i])
  return im


def read_single_digit(
  root_path: pathlib.Path,
  get_dataset: t.Callable[[], str | pathlib.Path],
  num: int,
  promise: bool,
) -> t.Callable[[], t.Iterator[pathlib.Path]]:
  """读取单个数字的所有图像
  Args:
      root_path: 根目录路径
      get_dataset: 获取数据集名称的函数
      num: 要读取的数字(0-9)
      promise: 是否确保路径存在
  Returns:
      返回一个生成器函数，用于遍历该数字的所有图像路径
  Raises:
      AssertionError: 当数字不在0-9范围内时抛出
      Exception: 当找不到图像时抛出
  """
  assert num in range(0, 10), "num must be 0~9"

  def might_fail_func() -> pathlib.Path:
    return path_fusion(root_path, str(get_dataset()), num)

  if promise:
    p = func.try_again(
      15,
      might_fail_func,
      is_validate_func=lambda p: p.exists(),
      fail_message=f"cannot num: {num}",
    )
  else:
    p = might_fail_func()

  logger.debug(f"path: {p}")

  def yield_pics():
    gen = files.scan_pics(path=p)
    try:
      img = next(gen)
      yield img
    except StopIteration:
      raise Exception(f"no images found in dataset {p}")

  return yield_pics



import abc


class Generator(abc.ABC):
  """所有生成器的入口，需要实现的生成函数"""

  @abc.abstractmethod
  def gen_func(self):
    pass


class Source(object):
  """"""

  pass


class ImageSource(Source):
  """原图像，定义清楚之后可以用于生成新的图像"""

  pass



from __future__ import annotations

import functools
import pathlib
import random
import sys
import typing as t

import toml
from loguru import logger

from meterviewer import T, files
from meterviewer.config import get_root_path
from meterviewer.datasets.v2 import dataset

# 设置控制台输出的日志级别为 WARNING
logger.remove()  # 移除默认的控制台输出
logger.add(sys.stdout, level="INFO")


getList = t.Literal["dataset", "path", "length", "total_nums"]


def load_config(config_path: pathlib.Path) -> t.Callable[[getList], t.Any]:
    data: t.Optional[dict] = None

    def load_conf() -> dict:
        nonlocal data
        if data is None:
            with open(config_path, "r") as f:
                data = toml.load(f)
        return data

    def get_config(name: str):
        c = load_conf().get("generate_config", None)
        if c is None:
            raise Exception('Config "generate_config" not found')
        value = c.get(name)
        return value

    def get_dataset_list() -> t.List[str]:
        dataset_list = get_config("dataset")
        if not isinstance(dataset_list, list):
            raise Exception(
                f"'dataset' should be a list, got {type(dataset_list)}: {dataset_list}"
            )
        return dataset_list

    def get_dataset() -> str:
        return random.choice(get_dataset_list())  # 随机选择一个数据集

    func_map = {
        "dataset": get_dataset,  # 随机选择的数据集
        "dataset_list": get_dataset_list,  # 返回完整数据集列表
        "path": functools.partial(get_config, name="path"),
        "length": functools.partial(get_config, name="length"),
        "total_nums": functools.partial(get_config, name="total_nums"),
    }

    def get_func(name: getList) -> t.Callable:
        return func_map[name]

    return get_func


def generate_dataset(config_path: pathlib.Path):
    root_path = get_root_path()
    get_f = load_config(config_path=config_path)
    print(get_f("dataset_list")())

    dataset_list = get_f("dataset_list")()

    # 生成数据集路径
    generated_path = root_path / pathlib.Path(get_f("path")())
    generated_path.mkdir(exist_ok=True)

    available_digits = dataset.scan_available_digits(root_path, dataset_list)

    # 创建生成逻辑
    create_dataset = dataset.create_dataset_func(check_imgs=lambda x: None)

    imgs, labels = create_dataset(
        root=root_path,
        dataset_list=dataset_list,
        available_digits=available_digits,
        nums=get_f("total_nums")(),
        gen_block_img=dataset.generate_block_img,  # 新逻辑
    )

    # 保存图像和标签
    files.save_img_labels_with_default(
        imgs, labels, prefix_name=generated_path, save_to_disk=files.save_to_disk
    )


def main():
    config_list = [
        "dataset-gen copy.toml",
        # "dataset-gen-2.toml",
        # "dataset-gen-3.toml",
        # "dataset-gen-4.toml",
        # "dataset-gen-5.toml",
    ]
    parent = pathlib.Path(__file__).parent
    for config in config_list:
        logger.info(f"generating dataset with {config}...")
        generate_dataset(parent / config)


if __name__ == "__main__":
    main()



import json
import os
import pathlib

import pytest

from meterviewer.generator.jsondb import gen_db, get_random_data
from meterviewer.generator.schema import MeterDB


@pytest.fixture
def gen(set_config):
  yield gen_db(infile=set_config, output=pathlib.Path(__file__).parent / "meterdb.json")
  os.unlink(pathlib.Path(__file__).parent / "meterdb.json")


def test_the_db(gen):
  assert gen.exists()


def test_db_content(gen):
  with open(gen, "r") as f:
    content = json.load(f)
  db = MeterDB.model_validate(content)
  assert len(db.data) > 0

  data = get_random_data()

  find = False
  for item in db.data:
    if item.filepath == data:
      find = True
      break
  assert find



from pathlib import Path as P

from meterviewer import files
from meterviewer import types as T
from meterviewer.datasets import dataset

from . import current


def more_quick_view(
  current_dataset: P,
  write_config=True,
):
  """增强的快速查看功能，自动检测数据集类型

  Args:
      current_dataset: 数据集路径
      write_config: 是否写入配置文件

  Returns:
      tuple: 包含X数据和Y数据的元组
  """
  name_func = files.use_smart_name(current_dataset)
  return quick_view(str(current_dataset), name_func, write_config)


def quick_view(
  current_dataset: str,
  get_x_y_name: T.NameFunc,
  write_config=True,
):
  """快速查看数据集

  Args:
      current_dataset: 数据集路径
      get_x_y_name: 获取X和Y文件名的函数
      write_config: 是否写入配置文件

  Returns:
      tuple: 包含X数据和Y数据的元组
  """
  x_name, y_name = get_x_y_name()
  x = files.load_from_disk(P(current_dataset) / x_name)
  y = files.load_from_disk(P(current_dataset) / y_name)

  current.view_merge_np(current_dataset, get_x_y=get_x_y_name)
  if write_config:
    current.write_details(current_dataset, get_xy_name=get_x_y_name)
  return x, y


def fast_preview(current_dataset: P):
  """最简单的数据集预览函数

  Args:
      current_dataset: 数据集路径

  Returns:
      tuple: 包含X数据和Y数据的元组
  """
  return more_quick_view(current_dataset)



from meterviewer.generator import db
from meterviewer.models import func
from pathlib import Path as P
import shutil
import pytest


def use_temp_db() -> P:
  dbpath = P("./alldata.db")
  new_db = dbpath.with_stem("alldata-temp")
  shutil.copy(dbpath, new_db)
  return new_db


def test_update_item():
  newdb = use_temp_db()
  func.update_item(newdb, 1, 2)

  item = func.get_first_item(newdb)
  assert item.carry_num == 2


@pytest.mark.skip("large database copy.")
def test_update_schema(remove_files):
  new_db = use_temp_db()

  func.update_schema(new_db)
  item = func.get_first_item(new_db)
  assert item.carry_num is None


def test_get_carry_items():
  items = func.get_carry_items(P("./alldata.db"))
  for item in items:
    assert item.is_carry == 1


def test_get_item():
  item = func.get_first_item(P("./alldata.db"))
  assert item.id == 1
